import icedata
from icevision.all import *


def test_parser(data_dir):
    parser = icedata.biwi.parser(data_dir)

    records = parser.parse(data_splitter=SingleSplitSplitter())[0]
    assert len(records) == 6
    record = records[3]

    expected = {
        "imageid",
        "labels",
        "bboxes",
        "filepath",
        "height",
        "width",
        "keypoints",
    }
    assert set(record.keys()) == expected

    assert record["filepath"].name == "3.jpg"
    assert record["imageid"] == 3
    assert record["labels"] == [1]
    assert record["height"] == 120
    assert record["width"] == 160

    expected = [BBox.from_xywh(0, 0, record["width"], record["height"])]
    assert record["bboxes"] == expected

    expected = KeyPoints.from_xyv([78, 63, 1], ["nose"])
    assert [
        (int(round(x)), int(round(y)), 1) for x, y, v in record["keypoints"][0].xyv
    ] == expected.xyv



__all__ = ["parser"]

from icevision.all import *


def parser(data_dir: Path):
    annotations_filepath = data_dir / "biwi_sample/centers.pkl"
    img_dir = data_dir / "biwi_sample/images/"

    return BIWIParser(annotations_filepath=annotations_filepath, img_dir=img_dir)


class BIWIKeypointsMetadata(KeypointsMetadata):
    labels = ["nose"]


class BIWIParser(
    parsers.Parser,
    parsers.FilepathMixin,
    parsers.KeyPointsMixin,
    parsers.LabelsMixin,
    parsers.BBoxesMixin,
):
    def __init__(self, annotations_filepath, img_dir):
        self.annotations_dict = pickle.load(open(Path(annotations_filepath), "rb"))
        self.img_dir = Path(img_dir)

    def __iter__(self):
        yield from self.annotations_dict.items()

    def __len__(self):
        return len(self.annotations_dict)

    def imageid(self, o):
        return int(o[0].replace(".jpg", ""))

    def filepath(self, o):
        return self.img_dir / o[0]

    def keypoints(self, o):
        y, x = o[1].tolist()
        return [KeyPoints.from_xyv([x, y, 1], BIWIKeypointsMetadata)]

    def image_width_height(self, o) -> Tuple[int, int]:
        return get_image_size(self.filepath(o))

    def labels(self, o) -> List[int]:
        return [1]

    def bboxes(self, o) -> List[BBox]:
        w, h = get_image_size(self.filepath(o))
        return [BBox.from_xywh(0, 0, w, h)]



__all__ = ["parser", "BirdsParser", "BirdMaskFile"]

from icevision.imports import *
from icevision.utils import *
from icevision.core import *
from icevision import parsers


def parser(data_dir: Union[str, Path], class_map: ClassMap) -> parsers.ParserInterface:
    return BirdsParser(data_dir=data_dir, class_map=class_map)


class BirdsParser(
    parsers.Parser,
    parsers.FilepathMixin,
    parsers.MasksMixin,
    parsers.BBoxesMixin,
    parsers.LabelsMixin,
):
    def __init__(self, data_dir, class_map):
        self.mat_filepaths = get_files(
            data_dir / "annotations-mat", extensions=[".mat"]
        )
        self.class_map = class_map

    def __iter__(self) -> Any:
        yield from self.mat_filepaths

    def __len__(self) -> int:
        return len(self.mat_filepaths)

    def filepath(self, o) -> Union[str, Path]:
        parts = list(o.parts)
        parts[-3] = "images"
        return Path(*parts).with_suffix(".jpg")

    def imageid(self, o) -> Hashable:
        return o.stem

    def image_width_height(self, o) -> Tuple[int, int]:
        return get_image_size(self.filepath(o))

    def masks(self, o) -> List[Mask]:
        return [BirdMaskFile(o)]

    def bboxes(self, o) -> List[BBox]:
        import scipy.io

        mat = scipy.io.loadmat(str(o))
        bbox = mat["bbox"]
        xyxy = [int(bbox[pos]) for pos in ["left", "top", "right", "bottom"]]
        return [BBox.from_xyxy(*xyxy)]

    def labels(self, o) -> List[int]:
        class_name = o.parent.name
        return [self.class_map.get_name(class_name)]


class BirdMaskFile(MaskFile):
    def to_mask(self, h, w):
        import scipy.io

        mat = scipy.io.loadmat(str(self.filepath))
        return MaskArray(mat["seg"])[None]



import icedata
from icevision.all import *


def test_parser(data_dir):
    parser = icedata.ochuman.parser(
        data_dir / "annotations/ochuman.json", data_dir / "images"
    )

    records = parser.parse(data_splitter=SingleSplitSplitter())[0]
    assert len(records) == 10
    record = records[3]

    expected = {
        "imageid",
        "labels",
        "bboxes",
        "filepath",
        "height",
        "width",
        "keypoints",
    }
    assert set(record.keys()) == expected

    assert record["filepath"].name == "000004.jpg"
    assert record["imageid"] == 3
    assert record["labels"] == [1, 1]
    assert record["height"] == 333
    assert record["width"] == 500
